import {getOneElementOrThrow} from "../libs/element.js";
import {tauriInvoke} from "../libs/tauri.js";
import {renderPanelFiles} from "./files/panel.js";
import {renderColumnFiles} from "./files/column.js";
import {renderListFiles} from "./files/list.js";
import {onceNavigate} from "../libs/navigation.js";

/**
 * @typedef {Object} Entry
 * @property {string} name
 * @property {string} path
 * @property {number} size
 * @property {boolean} is_directory
 * @property {boolean} is_file
 * @property {boolean} is_symlink
 * @property {string} permission
 * @property {number} directory_item_count
 * @property {number} created
 * @property {number} modified
 * @property {number} accessed
 */

let currentView = 'list';

export const renderFiles = async () => {
    const listEl = getOneElementOrThrow(document, '[data-ui=view-list]');
    const panelEl = getOneElementOrThrow(document, '[data-ui=view-panel]');
    const columnEl = getOneElementOrThrow(document, '[data-ui=view-column]');

    const handleClick = (toView) => () => {
        if (currentView !== toView) {
            currentView = toView;
            renderFiles();
        }
    }

    const once = true;
    listEl.addEventListener('click', handleClick('list'), {once});
    panelEl.addEventListener('click', handleClick('panel'), {once});
    columnEl.addEventListener('click', handleClick('column'), {once});

    switch (currentView) {
        case 'panel':
            renderPanelFiles();
            break;
        case 'column':
            renderColumnFiles();
            break;
        default:
            renderListFiles();
            break;
    }

    onceNavigate(() => renderFiles());
}

/**
 * @param {string} directory
 * @returns {Promise<Entry[]>}
 */
export const getEntries = async (directory) => {
    return await tauriInvoke('get_files', { directory });
}

/**
 * Convert bytes to a human-readable size format.
 *
 * @param {number} bytes - The number of bytes to convert.
 * @param {number} decimals - The number of decimal places to round to (optional, default is 2).
 * @returns {string} - A string representing the human-readable size.
 */
export const formatBytes = (bytes, decimals = 2) => {
    if (bytes === 0) return '0 Bytes';

    const k = 1024;
    const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];

    const i = Math.floor(Math.log(bytes) / Math.log(k));

    return parseFloat((bytes / Math.pow(k, i)).toFixed(decimals)) + ' ' + sizes[i];
}

export const formatDirectoryItemCount = (directoryItemCount) => {
    if (directoryItemCount === 1) {
        return '1 item'
    }

    return `${directoryItemCount} items`
}

export const openFile = async (path) => {
    return await tauriInvoke('open_file', { path });
}